import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { serializeJson } from "@modern-js/runtime-utils/node";
import { Await, useAsyncError } from "@modern-js/runtime-utils/router";
import { Suspense, useEffect, useMemo, useRef } from "react";
import { ROUTER_DATA_JSON_ID } from "../../core/constants";
import { defineRunInlineAndRouterDataInit, initRouterDataAttrs, runRouterDataFnStr, runWindowFnStr } from "./constants";
import { serializeErrors } from "./utils";
const DeferredDataScripts = (props) => {
  const staticContext = props === null || props === void 0 ? void 0 : props.context;
  const inlineScript = props === null || props === void 0 ? void 0 : props.inlineScript;
  const hydratedRef = useRef(false);
  useEffect(() => {
    hydratedRef.current = true;
  }, []);
  const deferredScripts = useMemo(() => {
    if (!staticContext) {
      return null;
    }
    const activeDeferreds = staticContext.activeDeferreds || [];
    const _ROUTER_DATA = {
      loaderData: staticContext.loaderData,
      errors: serializeErrors(staticContext.errors)
    };
    const initialScript0 = defineRunInlineAndRouterDataInit;
    const initialScript1 = inlineScript ? "" : `${serializeJson(_ROUTER_DATA)}`;
    const initialScript2 = {
      fnName: "initRouterData",
      fnArgs: [
        ROUTER_DATA_JSON_ID
      ],
      fnRun: runWindowFnStr,
      fnScriptSrc: "modern-run-window-fn"
    };
    const initialScript3 = initRouterDataAttrs;
    const deferredDataScripts = [];
    const initialScripts = Object.entries(activeDeferreds).map(([routeId, deferredData]) => {
      const pendingKeys = new Set(deferredData.pendingKeys);
      const { deferredKeys } = deferredData;
      const deferredKeyPromiseManifests = deferredKeys.map((key) => {
        if (pendingKeys.has(key)) {
          deferredDataScripts.push(/* @__PURE__ */ _jsx(DeferredDataScript, {
            nonce: props === null || props === void 0 ? void 0 : props.nonce,
            data: deferredData.data[key],
            dataKey: key,
            routeId
          }, `${routeId} | ${key}`));
          return {
            key,
            routerDataFnName: "s",
            routerDataFnArgs: [
              routeId,
              key
            ]
          };
        } else {
          const trackedPromise = deferredData.data[key];
          if (typeof trackedPromise._error !== "undefined") {
            const error = {
              message: trackedPromise._error.message,
              stack: process.env.NODE_ENV !== "production" ? trackedPromise._error.stack : void 0
            };
            return {
              key,
              routerDataFnName: "p",
              routerDataFnArgs: [
                void 0,
                serializeJson(error)
              ]
            };
          } else {
            if (typeof trackedPromise._data === "undefined") {
              throw new Error(`The deferred data for ${key} was not resolved, did you forget to return data from a deferred promise`);
            }
            return {
              key,
              routerDataFnName: "p",
              routerDataFnArgs: [
                serializeJson(trackedPromise._data)
              ]
            };
          }
        }
      });
      return {
        fnName: `mergeData`,
        fnRun: runWindowFnStr,
        fnArgs: [
          routeId,
          deferredKeyPromiseManifests
        ],
        fnScriptSrc: "modern-run-window-fn"
      };
    });
    return [
      initialScript0,
      initialScript1,
      initialScript2,
      initialScript3,
      initialScripts,
      deferredDataScripts
    ];
  }, []);
  if (!deferredScripts) {
    return null;
  }
  return /* @__PURE__ */ _jsxs(_Fragment, {
    children: [
      !hydratedRef.current && /* @__PURE__ */ _jsxs(_Fragment, {
        children: [
          /* @__PURE__ */ _jsx("script", {
            async: true,
            nonce: props === null || props === void 0 ? void 0 : props.nonce,
            "data-script-src": "modern-define-init",
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
              __html: deferredScripts[0]
            }
          }),
          deferredScripts[1].length !== 0 && /* @__PURE__ */ _jsx("script", {
            type: "application/json",
            id: ROUTER_DATA_JSON_ID,
            nonce: props === null || props === void 0 ? void 0 : props.nonce,
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
              __html: deferredScripts[1]
            }
          }),
          /* @__PURE__ */ _jsx("script", {
            async: true,
            nonce: props === null || props === void 0 ? void 0 : props.nonce,
            "data-script-src": deferredScripts[2].fnScriptSrc,
            "data-fn-name": deferredScripts[2].fnName,
            "data-fn-args": JSON.stringify(deferredScripts[2].fnArgs),
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
              __html: deferredScripts[2].fnRun
            }
          }),
          /* @__PURE__ */ _jsx("script", {
            async: true,
            nonce: props === null || props === void 0 ? void 0 : props.nonce,
            "data-script-src": "router-data-attr-init-inline",
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
              __html: deferredScripts[3]
            }
          }),
          deferredScripts[4].map(({ fnName, fnRun, fnArgs, fnScriptSrc }) => /* @__PURE__ */ _jsx("script", {
            async: true,
            "data-script-src": fnScriptSrc,
            "data-fn-name": fnName,
            "data-fn-args": JSON.stringify(fnArgs),
            nonce: props === null || props === void 0 ? void 0 : props.nonce,
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
              __html: fnRun
            }
          }, fnName))
        ]
      }),
      !hydratedRef.current && deferredScripts[5]
    ]
  });
};
const DeferredDataScript = ({ data, routeId, dataKey, nonce }) => {
  return /* @__PURE__ */ _jsx(Suspense, {
    children: typeof document === "undefined" && data && dataKey && routeId ? /* @__PURE__ */ _jsx(Await, {
      resolve: data,
      errorElement: /* @__PURE__ */ _jsx(ErrorDeferredDataScript, {
        routeId,
        dataKey,
        nonce
      }),
      children: (data2) => /* @__PURE__ */ _jsx("script", {
        async: true,
        nonce,
        "data-fn-name": "r",
        "data-script-src": "modern-run-router-data-fn",
        "data-fn-args": `${JSON.stringify([
          routeId,
          dataKey,
          data2
        ])}`,
        suppressHydrationWarning: true,
        dangerouslySetInnerHTML: {
          __html: runRouterDataFnStr
        }
      })
    }) : null
  });
};
const ErrorDeferredDataScript = ({ routeId, dataKey, nonce }) => {
  const error = useAsyncError();
  return /* @__PURE__ */ _jsx("script", {
    nonce,
    suppressHydrationWarning: true,
    dangerouslySetInnerHTML: {
      __html: `_ROUTER_DATA.r(${JSON.stringify(routeId)}, ${JSON.stringify(dataKey)}, ${void 0}, ${serializeJson({
        message: error.message,
        stack: error.stack
      })});`
    }
  });
};
var DeferredDataScripts_node_default = DeferredDataScripts;
export {
  DeferredDataScripts_node_default as default
};
